local frame = CreateFrame("FRAME", "ChatFrameClamping", UIParent); -- Need a frame to respond to events
frame:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded
frame:RegisterEvent("PLAYER_LOGOUT"); -- Fired when about to log out

local _G = getfenv(0);
local lastClamping;
local friendsMicroButton;
local gryphonsChar;
local TITLE = GetAddOnMetadata("ChatFrameClamping", "Title");
local VERSION = GetAddOnMetadata("ChatFrameClamping", "Version");
local AUTHOR = GetAddOnMetadata("ChatFrameClamping", "Author")

function frame:OnEvent(event, arg1)
	if event == "ADDON_LOADED" and arg1 == "ChatFrameClamping" then
		--print("|cffffd700" .. TITLE .. " |c0000cc00(|cffffffffv|c0000cc00" .. VERSION .. ")|cffc0c0c0 by " .. AUTHOR);

		if ChatFrameClampingChar == nil then
			ChatFrameClampingChar = "default";
			lastClamping = "default";
		end
		ChatFrameClamping_SlashCommand(ChatFrameClampingChar, false);

		if FriendsMicroButtonChar == nil then
			FriendsMicroButtonChar = false;
			friendsMicroButton = false;
		end
		setQuickJoinToastButton(FriendsMicroButtonChar);

		if GryphonsChar == nil then
			GryphonsChar = false;
			gryphonsChar = false;
		end
		setGryphons(GryphonsChar);

	elseif event == "PLAYER_LOGOUT" then -- Player is logging out or reloading ui
		ChatFrameClampingChar = lastClamping;
		FriendsMicroButtonChar = friendsMicroButton;
		GryphonsChar = gryphonsChar;
	end
end

frame:SetScript("OnEvent", frame.OnEvent);

SLASH_CHATFRAMECLAMPING1 = "/cfc";

--Enable the use of /SLASH_CHATFRAMECLAMPINGX commands
function SlashCmdList.CHATFRAMECLAMPING(value)
	ChatFrameClamping_SlashCommand(value, true);
end

function ChatFrameClamping_SlashCommand(value, showmsg)
	local rect, frame, clamp, doClamp;

	doClamp = true;

	if (value == "default") then
		-- Restore to game default
		-- Cannot move off screen or to the edge of the screen
		lastClamping = value;
		clamp = true;
		rect = true;
		if showmsg then print("Chat frames is restored to |c0000cc00game default") end

	elseif (value == "edge") then
		-- This option is enabled
		-- Can move to edge of screen
		lastClamping = value;
		clamp = true;
		rect = false;
		if showmsg then print("You can now move your chat frames to the |c0000cc00edge of the screen") end

	elseif (value == "offscreen") then
		-- This option is enabled
		-- Can move off screen
		lastClamping = value;
		clamp = false;
		rect = false;
		if showmsg then print("You can now move your chat frames |c0000cc00off screen") end

	elseif (value == "hidesocial") then
		doClamp = false;
		setQuickJoinToastButton(false);

	elseif (value == "showsocial") then
		doClamp = false;
		setQuickJoinToastButton(true);

	elseif (value == "gryphons") then
		doClamp = false;
		if gryphonsChar then
			setGryphons(false);
		else
			setGryphons(true);
		end

	elseif (value == "" or value == "help") then
		doClamp = false;
		print("|cffffd700" .. TITLE .. "|cffffffff commands:");
		print("/cfc");
		print(" default - Set to game default");
		print(" edge - Can move to the edge of the screen");
		print(" offscreen - Can move off screen");
		print(" hidesocial - Hide the social button that is attached to the chat window");
		print(" showsocial - Show the social button that is attached to the chat window");
		print(" gryphons - Show/hide the gryphons at the bottom bars");
		print(" status - Shows the current clamping status");
		print(" help - Shows this menu");

	elseif (value == "status") then
		doClamp = false;
		print("|cffffd700" .. TITLE .. "|cffffffff is set to |c0000cc00" .. lastClamping);

	else
		doClamp = false;
		print("'" .. value .. "' is not a valid command!");
		print("/cfc uses the commands 'default', 'toedge' and 'offscreen'");
		print(" default - Set to game default");
		print(" edge - Can move to the edge of the screen");
		print(" offscreen - Can move off screen");
		print(" hidesocial - Hide the social button that is attached to the chat window");
		print(" showsocial - Show the social button that is attached to the chat window");
		print(" gryphons - Show/hide the gryphons at the bottom bars");
	end

	if (doClamp) then
		for i = 1, NUM_CHAT_WINDOWS do
			frame = _G["ChatFrame"..i];
			frame:SetClampedToScreen(clamp);
			if (rect) then
				if (i == 1) then
					frame:SetClampRectInsets(-35, 35, 38, -50);
				else
					frame:SetClampRectInsets(-35, 35, 26, -50);
				end
			else
				frame:SetClampRectInsets(-1, 0, 26, 0);
			end
		end
	end
end

function setQuickJoinToastButton(status)
	local b = QuickJoinToastButton 
	b:UnregisterAllEvents() 
	if status then
		b:SetScript("OnShow", b.Show)
		b:Show()
		b.Toast2:Show()
		b.Toast:Show()
	else
		b:SetScript("OnShow", b.Hide)
		b:Hide()
		b.Toast2:Hide()
		b.Toast:Hide()
	end
	UIParent:UnregisterEvent("UNIT_AURA")

	friendsMicroButton = status;
end

function setGryphons(status)
	local gryph = MainMenuBarArtFrame

	if status then
		gryph.LeftEndCap:Hide()
		gryph.RightEndCap:Hide()
	else
		gryph.LeftEndCap:Show()
		gryph.RightEndCap:Show()
	end

	gryphonsChar = status;
end
